<?php

namespace Ourgold\Exchange\Services;

use Ourgold\Exchange\Models\ExchangeQueue;
use Illuminate\Support\Collection;

/**
 * Сервис для работы с задачами обмена с внешними сервисами в БД
 * Class ExchangeQueueStoreService
 * @package Ourgold\Exchange\Services
 */
class ExchangeQueueStoreService
{
    /**
     * Получить список не отправленных задач обмена
     * @return Collection|ExchangeQueue[]
     */
    public function getUnprocessed(): Collection
    {
        return ExchangeQueue::unprocessed()
            ->get();
    }

    public function createRaw(array $data): ExchangeQueue
    {
        return ExchangeQueue::query()->create(['data' => $data]);
    }

    /**
     * Создать или обновить задачу обмена
     * @param int $id
     * @param string|null $error
     * @return ExchangeQueue
     */
    public function update(int $id, ?string $error = null): ExchangeQueue
    {
        /** @var ExchangeQueue $item */
        $item = ExchangeQueue::query()->findOrFail($id);
        $item->update(
            [
                'status' => empty($error),
                'error' => $error
            ]
        );
        return $item;
    }
}
